#!/usr/bin/perl

=begin

	(filename)

	reform.plx

	(copyright)

	Copyright (c) 2013 Alexej Magura

	This file is part of Reform.

	Reform is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Reform is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Reform.  If not, see <http://www.gnu.org/licenses/>.

=cut

use File::Basename qw(basename);
use File::Temp;
use File::Compare;
use Getopt::Long;
use Text::Diff qw(diff);

Getopt::Long::Configure("bundling");

use strict; 
use warnings;

# Getopt Switch-Flags 

my $help=0;
my $version=0;
my $verbose=0;
my $printOnly=0;

# Getopt Value Vars

my $version_string='0.2.2';

GetOptions ("help|h" => \$help,
	    "version" => \$version,
	    "verbose|v" => \$verbose,
	    "dry-run|d|print-only|n" => \$printOnly)
    or exit 2;

#"target|T=s" => \$target,


if ( $help) {

    printf ( "usage: %s ", basename($0));
    print  ( "[option]... <file>... \n",
	     "\n",
	     "  -h, --help\t\tprint this message and exit\n",
	     "  -v, --verbose\t\tprint (unified) diff of the before and after file(s)\n",
	     "  -d, --dry-run\t\tprint, but do not commit, the after file(s)\n",
	     "      --version\t\tprint version and exit\n\n");

	 #"  -T, --target\t\tset the update target (default: startdir\n",
	 exit 0;

}
elsif ( $version) {

    printf ( "%s\n", $version_string);
    exit 0;
}



sub main {


    my $PREPARE=0;
    my $BUILD=0;
    my $PACKAGE=0;
    my $fh = File::Temp->new( SUFFIX => '.tmp', TEMPLATE => 'reform-XXXXXXX', DIR => '/tmp', UNLINK => 0); 
    my $fname = $fh->filename;

    if ( not ( @_)) {

	exit 3;

    }

    foreach my $file (@_) {

	if ( not (-e $file)) {

	    exit 1;
	}
	elsif ( not (-R $file)) {

	    exit 5;

	}

	open ( FILE_a, "<$file");
	open ( FILE_b, ">$fname");


	while (<FILE_a>) {

	    if ( $_ =~ /^\s*build\s*\(\)/ ) {

		$BUILD=1; 

	    }

	    if ( $_ =~ /^\s*package\s*\(\)/ ) {

		$PACKAGE=1;

	    }

	    if ( $_ =~ /^\s*prepare\s*\(\)/ ) {

		$PREPARE=1;

	    }

	}

	seek(FILE_a, 0, 0);

	while (<FILE_a>) {

	    if ( $BUILD and (not $PACKAGE)) {


		# replace build()|build () with package()|package ()

		$_ =~ s/^(\h*)build(\s*)\(\)/$1package$2\(\)/;

	    }
		    
=begin 
	    Replace all instances of `$startdir/(pkg|src)' 
	    with their respective updated forms, that is, `$srcdir' and `$pkgdir'

=cut
	    $_ =~ s/\$startdir\/src(\/?)/\$srcdir$1/g;
	    $_ =~ s/\$startdir\/pkg(\/?)/\$pkgdir$1/g;

	    # get rid of all the `|| return 1' statements in PKGBUILD 

	    $_ =~ s/^(.*)\h*\|\|\h*return\h*1\h*(.*)$/$1 $2/g;

	    #switch ( $target) {

	    #	case "startdir" {
		 

	    #	}

	    # }

	    print FILE_b "$_";

	}

	close(FILE_a);
	close(FILE_b);

	if ( compare($file, $fname) == 0) {

	    unlink $fname;

	    exit 4;
	}
	
	open(FILE_b, "<$fname");

	if ( $verbose) {

	    print diff $file => $fname;


	}

	if ( $verbose and $printOnly) {

	    print "\e[0;34m~\e[0;0m\n" x 10;
	}
	
	if ( $printOnly) { 

	    while (<FILE_b>) {

		print "$_";

	    }

	    unlink $fname;

	    exit 0;

	}

	open(FILE_a, ">$file");

	while (<FILE_b>) {

	    print FILE_a "$_";
	
	}

	close(FILE_a);
	close(FILE_b);


	

	unlink $fname;

    }
}

main(@ARGV);
